package top.ninwoo;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.bishe.starter.entity.ContainerMonitor;
import top.ninwoo.bishe.starter.service.ClusterService;
import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.common.entity.*;
import top.ninwoo.test.BisheTestMain;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = BisheTestMain.class)
public class BisheMultiNodeTests {

    @Autowired
    private ClusterService clusterService;

    @Resource
    private NetworkService networkService;

    @Test
    public void testNetworkChange() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        separatedClusterConfig.setEdgeNodeId("192.168.61.130:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("APP1");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"APP1", "br:ovs1", "br:ovs2", "br:remote:ovs3:192.168.61.131"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{
                {0,0,0,0},
                {1,0,0,0},
                {0,1,0,0},
                {0,0,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        // 第二个节点
        SeparatedClusterConfig separatedClusterConfig1 = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig1.setEdgeNodeId("192.168.61.131:8081");
        ClusterConfig clusterConfig1 = new ClusterConfig();
        clusterConfig1.setId(11111l);
        clusterConfig1.setOwner("joliu");
        ContainerDescription containerDescription11 = new ContainerDescription();
        containerDescription11.setMode("normal");
        containerDescription11.setReplicas(1);
        DockerContainer container11 = new DockerContainer();
        container11.setName("APP3");
        container11.setCommand("sh");
        container11.setImage("joliu/networktest");
        containerDescription11.setDockerContainer(container11);

        List<ContainerDescription> cds1 = new ArrayList<>();
        cds1.add(containerDescription11);

        ContainerDescription containerDescription12 = new ContainerDescription();
        containerDescription12.setMode("normal");
        containerDescription12.setReplicas(1);
        DockerContainer container12 = new DockerContainer();
        container12.setName("APP4");
        container12.setCommand("sh");
        container12.setImage("joliu/networktest");
        containerDescription12.setDockerContainer(container12);

        cds1.add(containerDescription12);

        clusterConfig1.setDockers(cds1);
        NetworkTopology topo1 = new NetworkTopology();
        topo1.setAppNames(new String[]{"APP2", "br:ovs3", "br:ovs4","br:ovs5","br:ovs6","br:ovs7","br:ovs8","br:remote:ovs2:192.168.61.130"});

        // 这个参数好像没啥用
        topo1.setTopologyId(11);
        topo1.setTopology(new int[][]{{0,0,0,0,0,0,0,0}
                                     ,{0,0,0,0,0,0,0,0}
                                     ,{0,1,0,0,0,0,0,0}
                                     ,{0,0,1,0,0,0,0,0}
                                     ,{0,0,1,1,0,0,0,0}
                                     ,{0,0,0,0,1,0,0,0}
                                     ,{1,0,0,0,0,1,0,0}
                                     ,{0,1,0,0,0,0,0,0}});
        clusterConfig1.setTopology(topo1);
        separatedClusterConfig1.setClusterConfig(clusterConfig1);
        clusterConfigs.add(separatedClusterConfig1);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }


    @Test
    public void testSendSeparateConfig() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.61.130:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("APP1");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription containerDescription1 = new ContainerDescription();
        containerDescription1.setMode("normal");
        containerDescription1.setReplicas(1);
        DockerContainer container1 = new DockerContainer();
        container1.setName("APP2");
        container1.setCommand("sh");
        container1.setImage("joliu/networktest");
        containerDescription1.setDockerContainer(container1);

        cds.add(containerDescription1);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"APP1", "APP2", "br:ovs1", "br:remote:ovs2:192.168.61.131"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{{0,0,0,0},{0,0,0,0},{1,1,0,0},{0,0,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        // 构建第二个节点
        SeparatedClusterConfig separatedClusterConfig1 = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig1.setEdgeNodeId("192.168.61.131:8081");
        ClusterConfig clusterConfig1 = new ClusterConfig();
        clusterConfig1.setId(11111l);
        clusterConfig1.setOwner("joliu");
        ContainerDescription containerDescription11 = new ContainerDescription();
        containerDescription11.setMode("normal");
        containerDescription11.setReplicas(1);
        DockerContainer container11 = new DockerContainer();
        container11.setName("APP3");
        container11.setCommand("sh");
        container11.setImage("joliu/networktest");
        containerDescription11.setDockerContainer(container11);

        List<ContainerDescription> cds1 = new ArrayList<>();
        cds1.add(containerDescription11);

        ContainerDescription containerDescription12 = new ContainerDescription();
        containerDescription12.setMode("normal");
        containerDescription12.setReplicas(1);
        DockerContainer container12 = new DockerContainer();
        container12.setName("APP4");
        container12.setCommand("sh");
        container12.setImage("joliu/networktest");
        containerDescription12.setDockerContainer(container12);

        cds1.add(containerDescription12);

        clusterConfig1.setDockers(cds1);
        NetworkTopology topo1 = new NetworkTopology();
        topo1.setAppNames(new String[]{"APP3", "APP4", "br:ovs2", "br:remote:ovs1:192.168.61.130"});
        // 这个参数好像没啥用
        topo1.setTopologyId(11);
        topo1.setTopology(new int[][]{{0,0,0,0},{0,0,0,0},{1,1,0,0},{0,0,1,0}});
        clusterConfig1.setTopology(topo1);
        separatedClusterConfig1.setClusterConfig(clusterConfig1);
        clusterConfigs.add(separatedClusterConfig1);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    @Test
    public void removeClusterFromEdgeNodeTest() {
        clusterService.removeClusterFromEdgeNode(11111L);
    }

    @Test
    public void getLogicalNetworkTopologyTest() {
        NetworkTopology logicalNetworkTopology = networkService.getLogicalNetworkTopology(11111L);
        System.out.println(logicalNetworkTopology);
    }

    @Test
    public void getIpListByAppNameTest() {
        List<String> ipList = networkService.getIpListByAppName(11111L, "APP3");
        System.out.println(ipList);
    }

    @Test
    public void enableNetworkMonitorTest() throws InterruptedException {
        String res = networkService.enableNetworkMonitor(11111L, "Run");
        System.out.println(res);
    }

    @Test
    public void cancelNetworkMonitorTest() {
        String run = networkService.cancelNetworkMonitor(11111L, "Run");
        System.out.println(run);
    }

    @Test
    public void getNetworkMonitorTest() throws InterruptedException {
        List<ContainerMonitor> run = networkService.getContainerMonitors(11111L, "Run");
        int i = 0;
        while(i < 10) {
            run.forEach(c -> {
                System.out.println(c.getContainerInfo());
            });
            Thread.sleep(500);
            i++;
        }
    }

    @Test
    public void getAllEdgeNodeIdsTest() {
        List<String> allEdgeNodeIds = clusterService.getAllEdgeNodeIds();
        System.out.println(allEdgeNodeIds);
    }
}
